@@ -12,6 +12,7 @@
 # limitations under the License.
 
 import itertools
+import logging
 import uuid
 
 from django.utils import encoding
@@ -39,9 +40,24 @@ from sahara_dashboard.content.data_processing.utils \
 from sahara_dashboard.content.data_processing.utils \
     import workflow_helpers
 
+LOG = logging.getLogger(__name__)
+
 BASE_IMAGE_URL = "horizon:project:data_processing.clusters:register"
 
 
+def is_cinder_enabled(request):
+    return saharaclient.base.is_service_enabled(request, 'volume')
+
+
+def storage_choices(request):
+    choices = [("ephemeral_drive", _("Ephemeral Drive")), ]
+    if is_cinder_enabled(request):
+        choices.append(("cinder_volume", _("Cinder Volume")))
+    else:
+        LOG.warning(_("Cinder service is unavailable now"))
+    return choices
+
+
 class GeneralConfigAction(workflows.Action):
     nodegroup_name = forms.CharField(label=_("Template Name"))
 
@@ -61,8 +77,7 @@ class GeneralConfigAction(workflows.Action):
     storage = forms.ChoiceField(
         label=_("Storage location"),
         help_text=_("Choose a storage location"),
-        choices=[("ephemeral_drive", "Ephemeral Drive"),
-                 ("cinder_volume", "Cinder Volume")],
+        choices=[],
         widget=forms.Select(attrs={
             "class": "storage_field switchable",
             'data-slug': 'storage_loc'
@@ -180,22 +195,28 @@ class GeneralConfigAction(workflows.Action):
             widget=forms.HiddenInput(),
             initial=hadoop_version
         )
+
+        self.fields["storage"].choices = storage_choices(request)
+
         node_parameters = hlps.get_general_node_group_configs(plugin,
                                                               hadoop_version)
         for param in node_parameters:
             self.fields[param.name] = workflow_helpers.build_control(param)
 
-        if request.REQUEST.get("guide_template_type"):
+        # when we copy or edit a node group template then
+        # request contains valuable info in both GET and POST methods
+        req = request.GET.copy()
+        req.update(request.POST)
+        if req.get("guide_template_type"):
             self.fields["guide_template_type"] = forms.CharField(
                 required=False,
                 widget=forms.HiddenInput(),
-                initial=request.REQUEST.get("guide_template_type"))
+                initial=req.get("guide_template_type"))
 
-        try:
+        if is_cinder_enabled(request):
             volume_types = cinder.volume_type_list(request)
-        except Exception:
-            exceptions.handle(request,
-                              _("Unable to get volume type list."))
+        else:
+            volume_types = []
 
         self.fields['volume_type'].choices = [(None, _("No volume type"))] + \
                                              [(type.name, type.name)
@@ -217,9 +238,10 @@ class GeneralConfigAction(workflows.Action):
 
     def populate_volumes_availability_zone_choices(self, request, context):
         az_list = [(None, _('No availability zone specified'))]
-        az_list.extend([(az.zoneName, az.zoneName)
-                        for az in cinder_utils.availability_zone_list(request)
-                        if az.zoneState['available']])
+        if is_cinder_enabled(request):
+            az_list.extend([(az.zoneName, az.zoneName)
+                            for az in cinder_utils.availability_zone_list(
+                                request) if az.zoneState['available']])
         return az_list
 
     def populate_image_choices(self, request, context):
@@ -228,10 +250,14 @@ class GeneralConfigAction(workflows.Action):
 
     def get_help_text(self):
         extra = dict()
-        plugin, hadoop_version = (
+        plugin_name, hadoop_version = (
             workflow_helpers.get_plugin_and_hadoop_version(self.request))
-        extra["plugin_name"] = plugin
+        extra["plugin_name"] = plugin_name
         extra["hadoop_version"] = hadoop_version
+        plugin = saharaclient.plugin_get_version_details(
+            self.request, plugin_name, hadoop_version)
+        extra["deprecated"] = workflow_helpers.is_version_of_plugin_deprecated(
+            plugin, hadoop_version)
         return super(GeneralConfigAction, self).get_help_text(extra)
 
     class Meta(object):
@@ -434,12 +460,9 @@ class ConfigureNodegroupTemplate(workflow_helpers.ServiceParametersWorkflow,
         plugin, hadoop_version = (
             workflow_helpers.get_plugin_and_hadoop_version(request))
 
-        general_parameters = hlps.get_general_node_group_configs(
-            plugin,
-            hadoop_version)
-        service_parameters = hlps.get_targeted_node_group_configs(
-            plugin,
-            hadoop_version)
+        general_parameters, service_parameters = \
+            hlps.get_general_and_service_nodegroups_parameters(plugin,
+                                                               hadoop_version)
 
         if saharaclient.base.is_service_enabled(request, 'share'):
             ConfigureNodegroupTemplate._register_step(self,
